/**
   GeoShift (C) 2018 Beholder Corporation / FoxyProxy
 * support@getfoxyproxy.org
 *
 * This source code is proprietary, and released under the EULA available in the
 * LICENSE file at the root of this installation. Do not copy or re-use without
 * written permission.
 */

const SSL_PORT = 4443;
const NO_SSL_PORT = 13129;

function localizeHtmlPage()
{
    //Localize by replacing __MSG_***__ meta tags
    var objects = document.getElementsByTagName('html');
    for (var j = 0; j < objects.length; j++)
    {
        var obj = objects[j];

        var valStrH = obj.innerHTML.toString();
        var valNewH = valStrH.replace(/__MSG_(\w+)__/g, function(match, v1)
        {
            return v1 ? chrome.i18n.getMessage(v1) : "";
        });

        if(valNewH != valStrH)
        {
            obj.innerHTML = valNewH;
        }
    }
}
$(document).foundation();
let DIRECT = {value: {mode: 'direct'}, scope: 'regular'}, username, password;
$(document).ready(function() {
    localizeHtmlPage();
    chrome.storage.sync.get(null, (items) => {
        if ("credentials" in items) {
          username = items.credentials.username;
          password = items.credentials.password;
          $("#username").val(items.credentials.username);
          $("#password").val(items.credentials.password);
        }
        else {
          // No credentials. Show login page.
          $("#loginRow").show();
          $("#accountsRow").hide(); 
          return;
        }
        if (items.accounts) {
            console.log("found accounts");
            populateAccounts(items.accounts);
            $("#loginRow").hide();
            $("#accountsRow").show();            
        }
        else {
            // force login again
            $("#loginRow").show();
            $("#accountsRow").hide();
        }

        if (items.useSSL) {
        console.log("useSSL " + items.useSSL);
         $('#useSSL').prop('checked', true);
        }
    });
});

$(document).ready(function() {
    $('body').on('click', 'a', function(){
     chrome.tabs.create({url: $(this).attr('href')});
     return false;
    });
});

$('#useSSL').on('click', function(e) {
    let val = $(this).prop('checked');
    chrome.storage.sync.set({'useSSL': val}, function() {
        console.log("Stored new useSSL value: " + val);
        // Are we currently using a proxy server? Is so, update the port and scheme
        chrome.proxy.settings.get({'incognito': false}, function(config) {
            console.log("current proxy settings: " + JSON.stringify(config));
            if (config.levelOfControl == "controlled_by_other_extensions") {
                console.log("controlled by other extension");
                showMessage("<strong>Another Chrome extension is controlling proxy settings.<br>Please disable it.</strong>", true);
                return;
            }
            let currentServer = config.value.mode == "fixed_servers" ? config.value.rules.proxyForHttp.host : '',
                currentPort = config.value.mode == "fixed_servers" ? config.value.rules.proxyForHttp.port : '';
            console.log("current server is " + currentServer);
            if (currentServer) {
                // It's on
                let proxyConfig = {scheme: val ? 'https' : 'http', host: currentServer, port: val ? SSL_PORT : NO_SSL_PORT};
                console.log("Using: " + JSON.stringify(proxyConfig));
                let config = {
                    mode: 'fixed_servers',
                    rules: {
                        proxyForHttp: proxyConfig,
                        proxyForHttps: proxyConfig,
                        proxyForFtp: proxyConfig,
                        fallbackProxy: proxyConfig,
                        // TODO: add all local addresses
                        bypassList: ["<local>", "10.0.0.0/8", "169.254.0.0/16", "172.16.0.0/12", "192.168.0.0/16", "getfoxyproxy.org"]
                    }
                };
                // No need to update credentials--they didn't change
                chrome.proxy.settings.set({value: config, scope: 'regular'}, function() {});
            }
        });
    });
});


$('#login-button').on('click', function(e) {
    $('#login-error').hide();
    $('#login-button').prop('disabled', true);

    // Custom ajax request.
    // params: spinnerSelector, btnSelector, url, successCallbackAndParams, params (optional)
    // successCallbackAndParams is null or an object with two props: callback and params
    // errorCallbackAndParams is null or an object with two props: callback and params
    // URL parameters can be passed directly as part of |url| or as an object of key/value pairs in |params|; example:
    // {url: 'http://foo.com', params: {'key1': 'value1', 'key2': 'value2'}} becomes http://foo.com?key1=value1&key2=value2.
    // This is standard jquery behavior, nothing new.
    // You can also just do:
    // {url: 'http://foo.com?key1=value1&key2=value2'}
    var credentials = {username: $("#username").val(), password: $("#password").val()};
    act({params: credentials, spinnerSelector: '#spinner', btnSelector: '#login-button', url: 'https://getfoxyproxy.org/webservices/get-accounts.php',
        successCallbackAndParams: {callback: loginSuccess, params: credentials}, errorCallbackAndParams: {callback: loginFailure, params: credentials}});
});

function loginSuccess(json, credentials) {
    for (var i in json) {
        var entry = json[i];
        delete entry.ovpnFile; // We don't need this and if we store it, we easily get QUOTA_BYTES_PER_ITEM errors
        delete entry.hasPPTP; // We don't need this 
    }
    console.log('about to store ' + JSON.stringify(json));
    chrome.storage.sync.set({'credentials': credentials}, function() { // Try storing the credentials separately in case storing of accounts fails (that happens when there are too many of them)
        chrome.storage.sync.set({'accounts': json}, function() {
            // extensions::lastError:131 Unchecked runtime.lastError while running storage.set: QUOTA_BYTES_PER_ITEM quota exceeded
            // at Object.loginSuccess [as callback] (chrome-extension://bhndceanmnpgmpghhednjgapbfjlcnjk/scripts/popup.js:123:25)

            // If there is a selected proxy, check that the server/port are still valid
            //chrome.storage.sync.get('selected', function(config) {
                //if (config && config.proxyForHttp && (config.proxyForHttp !=
            //});

            if (chrome.runtime.lastError) {
                showMessage("We were unable to store your accounts.<br>You need to login again next time you open GeoShift.", true);
                $('#error-row').show();
            }
            $('#viewaccounts-button').show();
            $("#loginRow").slideUp(200, function() {
                populateAccounts(json);
                $("#accountsRow").slideDown();
            });
        });
    });
}

function loginFailure(json, credentials) {
    $('#login-error').show();
    $('#viewaccounts-button').hide();
    chrome.storage.sync.set({'credentials': credentials}, function() {
        chrome.storage.sync.remove('accounts'); // In case they'd previously worked
        //chrome.storage.sync.remove('selectedConfig');
    });
}

$('#refresh-button').on('click', function() {
    chrome.storage.sync.get('credentials', function(obj) {
        $('#viewaccounts-button').show();
        $('#username').val(obj && obj.credentials && obj.credentials.username);
        $('#password').val(obj && obj.credentials && obj.credentials.password);
        $("#loginRow").show();
        $("#accountsRow,#error-row").hide();
    });
});

$('#viewaccounts-button').on('click', function() {
    $("#loginRow").slideUp(200, function() {
        $("#accountsRow").slideDown();
    });    
});

$('#disableall-button').on('click', function() {
    chrome.proxy.settings.set(DIRECT,
        function() {
            chrome.storage.sync.remove('accounts', function() {
                offButton($('input[data-server]'));
                chrome.runtime.getBackgroundPage(function(page) {
                    page.setCredentials({'authCredentials': null});
                });
            });            
        }
    );
});


$('#accountsContainer').on('click', 'input[data-server]', function() {
    let useSSL = $('#useSSL').prop('checked');
    let scheme = useSSL ? 'https' : 'http';

    let server = $(this).attr('data-server'), port = $(this).attr('data-port'),
        username = $(this).attr('data-username'),
        password = $(this).attr('data-password'),
        ttt = $(this),
        proxyConfig = {scheme: scheme, host: server, port: useSSL ? SSL_PORT : parseInt(port)};
    if (ttt.attr('selected') == 'selected') {
        // It's already on. Turn it off.
        chrome.proxy.settings.set(DIRECT);
        offButton(ttt);
    }
    else {
        // It's already off. Turn it on. TODO: but only if nothing else is already on
        var config = {
            mode: 'fixed_servers',
            rules: {
                proxyForHttp: proxyConfig,
                proxyForHttps: proxyConfig,
                proxyForFtp: proxyConfig,
                fallbackProxy: proxyConfig,
                // TODO: add all local addresses
                bypassList: ["<local>", "10.0.0.0/8", "169.254.0.0/16", "172.16.0.0/12", "192.168.0.0/16"]
            }
        };
        console.log("Using: " + JSON.stringify(proxyConfig));
        chrome.proxy.settings.set({value: config, scope: 'regular'},
            function() {
                chrome.runtime.getBackgroundPage(function(page) {
                    page.setCredentials({'authCredentials': {'username': username, 'password': password}});
                });
                onButton(ttt);
                chrome.storage.sync.set({'credentials': {username: username, password: password}}, function() {
                    console.log('selected');
                });
            }
        );
        //chrome.storage.sync.set({'selectedConfig': config});
    }
});

function populateAccounts(accounts) {
    // Find the proxy that's currently in use so we can highlight it if its in this list
    chrome.proxy.settings.get({'incognito': false}, function(config) {
        console.log("current proxy settings: " + JSON.stringify(config));
        if (config.levelOfControl == "controlled_by_other_extensions") {
            console.log("controlled by other extension");
            showMessage("<strong>Another Chrome extension is controlling proxy settings.<br>Please disable it.</strong>", true);
            return;
        }
        let currentServer = config.value.mode == "fixed_servers" ? config.value.rules.proxyForHttp.host : '';
        console.log("current server is " + currentServer);
        var tmp = [], template = $('#accountRowTemplate').html(), 
            noCountryTemplate = $('#accountRowTemplateNoNode').html(),
            somethingWillBeSelected = false, allInactive = true,
            oneOftheUsernames, oneOfthePasswords;
        for (var i=0; i<accounts.length; i++) {
            // Find if there's one that's going to be selected
            if (currentServer && currentServer == accounts[i].hostname) {
                somethingWillBeSelected = true;
                console.log(currentServer + " will be selected");
            }
        }
        for (var i=0; i<accounts.length; i++) {
            account = accounts[i];
            if (!account.country) {
                oneOftheUsernames = account.username;
                oneOfthePasswords = account.password;
                var t = noCountryTemplate.replace(/%data-username/g, account.username);
                t = t.replace(/%data-password/g, account.password);
                tmp.push(t);
                continue;
            }
            if (account.active === "true")
                allInactive = false;
            var t = template.replace(/%data-country/g, account.country);
            t = t.replace(/%data-server/g, account.hostname);
            t = t.replace(/%data-ip/, account.ip);
            // Customers with more than one account: their username/password is likely different for each account.
            t = t.replace(/%data-username/g, account.username);
            t = t.replace(/%data-password/g, account.password);
            oneOftheUsernames = account.username;
            oneOfthePasswords = account.password;
            // TODO: deal with ports as array
            t = t.replace(/%data-port/g, account.port);
            if (account.country_code.toLowerCase() == "uk") account.country_code = "gb";
            t = t.replace(/%data-cc/g, account.country_code.toLowerCase());
            if (currentServer == account.hostname) {
                t = t.replace('%color', account.active == 'true' ? 'success' : 'secondary');
                t = t.replace('%selected', 'selected');
                t = t.replace('%onOrOff', 'Turn Off');
                // Normally, we'd always hide the Turn On/Off button if the account is inactive.
                // But if this node is selected and the account
                // inactive, we must give the user a way to turn it off anyway.
                // Commenting-out for now. Causing too many problems when customer has 2 accounts on same node, one active and one inactive (different usernames, of course)
                // t = t.replace('%hideIfInactive', '');
                t = t.replace('%hideIfActive', account.active == 'true' ? 'hide' : '');
                t = t.replace('%hideIfInactive', account.active == 'true' ? '' : 'hide');
                tmp.unshift(t); // Place at top
            }
            else {
                t = t.replace('%hideIfActive', account.active == 'true' ? 'hide' : '');
                t = t.replace('%hideIfInactive', account.active == 'true' ? '' : 'hide');
                t = t.replace('%color', account.active == 'true' ? '' : 'secondary');
                t = t.replace('%selected', somethingWillBeSelected ? 'disabled' : ''); // Disable btn if something is selected; otherwise make it selectable
                t = t.replace('%onOrOff', 'Turn On');
                tmp.push(t);
            }                
        }
        if (allInactive) {
            showMessage('You have no active accounts.<br>You can renew them <a class="renew-link">here</a> or buy new ones <a href="https://getfoxyproxy.org/order/">here</a>.', true);
        }
        $("#accounts").html(tmp.join(''));

        // Keep next line AFTER $("#accounts").html() otherwise the .renew-link nodes aren't in the DOM yet
        $('.renew-link').attr('href', 'https://getfoxyproxy.org/panel/?username=' + oneOftheUsernames + 
            '&password=' + oneOfthePasswords);
    });
}

function offButton(btn) {
    btn.attr('value', 'Turn On');
    btn.removeAttr('selected');
    $('.callout').removeClass('success'); // Applies to ALL rows
    $('input[data-server]').prop('disabled', false);
    chrome.browserAction.setBadgeText({text: ''});
    chrome.browserAction.setTitle({title: ''});
    //chrome.browserAction.setBadgeBackgroundColor({color: ''}); // error
    chrome.browserAction.setIcon({
        path: {
            "16": "images/blue-globe-foxy-alt-icon-64.png",
            "32": "images/blue-globe-foxy-alt-icon-128.png"
        }
    }, genericCallbackErrorHandler);
}

function onButton(selector) {
    if (typeof selector === 'string') selector = $(selector);
    selector.attr('value', 'Turn Off');
    selector.attr('selected', 'true');
    selector.closest('.callout').addClass('success'); // The green glow for the row
    // Disable all buttons...
    $('input[data-server]').prop('disabled', true);
    // ... except the one that was clicked
    selector.prop('disabled', false);
    var countryCode = selector.attr('data-cc'),
        flagPath = 'images/flagicon-2.4.0/1x1/' + countryCode + '.svg';
    //console.log("country code is " + countryCode);
    //console.log(flagPath);
    chrome.browserAction.setBadgeText({text: countryCode});
    chrome.browserAction.setTitle({title: selector.attr('data-country')});
    chrome.browserAction.setBadgeBackgroundColor({color: '#cc6600'});
    var flagIcon32 = document.getElementById('flag-icon-canvas-32'), ctx32 = flagIcon32.getContext('2d'),
            flagIcon16 = document.getElementById('flag-icon-canvas-16'), ctx16 = flagIcon16.getContext('2d');

    var img = new Image();
    img.src = chrome.extension.getURL(flagPath);
    img.onload = function() {
        ctx32.clearRect(0, 0, 32, 32);
        ctx16.clearRect(0, 0, 16, 16);
        ctx32.drawImage(img, 0, 0, 32, 32);
        ctx16.drawImage(img, 0, 0, 16, 16);
        var imageData32 = ctx32.getImageData(0, 0, 32, 32), imageData16 = ctx16.getImageData(0, 0, 16, 16);
        chrome.browserAction.setIcon({
            imageData: {
                "16": imageData16,
                "19": imageData16,
                "32": imageData32,
                "38": imageData32
            }
        }, genericCallbackErrorHandler);   
    }
    img.src = chrome.extension.getURL(flagPath);
}

function showMessage(textOrHtml, isHtml) {
  if (isHtml) $('#errorContent').html(textOrHtml);
  else $('#errorContent').text(textOrHtml);
  $('#error-row').show();
}

// To prevent exceptions being thrown, which prevents subsequent JS from being executed
function genericCallbackErrorHandler() {
    if (chrome.runtime.lastError)
        console.log("There was an error: " + chrome.runtime.lastError.message);
}

$('#accountsContainer').on('click', '#close-button', (e) => {
    $("#error-row").slideUp(100);
});

